@@ -13,10 +13,25 @@ func TestConfigAnalyzer_Required(t *testing.T) {
 		want     bool
 	}{
 		{
-			name:     "happy path",
+			name:     "tf",
 			filePath: "/path/to/main.tf",
 			want:     true,
 		},
+		{
+			name:     "tf.json",
+			filePath: "/path/to/main.tf.json",
+			want:     true,
+		},
+		{
+			name:     "tfvars",
+			filePath: "/path/to/some.tfvars",
+			want:     true,
+		},
+		{
+			name:     "json",
+			filePath: "/path/to/some.json",
+			want:     false,
+		},
 		{
 			name:     "hcl",
 			filePath: "/path/to/main.hcl",
@@ -27,6 +42,11 @@ func TestConfigAnalyzer_Required(t *testing.T) {
 			filePath: "deployment.yaml",
 			want:     false,
 		},
+		{
+			name:     "manifest snapshot file",
+			filePath: ".terraform/modules/modules.json",
+			want:     true,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
