@@ -1,10 +1,11 @@
 package applier
 
 import (
+	"github.com/samber/lo"
 	"golang.org/x/xerrors"
 
+	"github.com/aquasecurity/trivy/pkg/cache"
 	"github.com/aquasecurity/trivy/pkg/fanal/analyzer"
-	"github.com/aquasecurity/trivy/pkg/fanal/cache"
 	ftypes "github.com/aquasecurity/trivy/pkg/fanal/types"
 )
 
@@ -23,11 +24,15 @@ func NewApplier(c cache.LocalArtifactCache) Applier {
 
 func (a *applier) ApplyLayers(imageID string, layerKeys []string) (ftypes.ArtifactDetail, error) {
 	var layers []ftypes.BlobInfo
+	var layerInfoList ftypes.Layers
 	for _, key := range layerKeys {
 		blob, _ := a.cache.GetBlob(key) // nolint
 		if blob.SchemaVersion == 0 {
 			return ftypes.ArtifactDetail{}, xerrors.Errorf("layer cache missing: %s", key)
 		}
+		if l := blob.Layer(); !lo.IsEmpty(l) {
+			layerInfoList = append(layerInfoList, l)
+		}
 		layers = append(layers, blob)
 	}
 
@@ -40,6 +45,9 @@ func (a *applier) ApplyLayers(imageID string, layerKeys []string) (ftypes.Artifa
 		Secret:           imageInfo.Secret,
 	}
 
+	// Fill layers info
+	mergedLayer.Layers = layerInfoList
+
 	if !mergedLayer.OS.Detected() {
 		return mergedLayer, analyzer.ErrUnknownOS // send back package and apps info regardless
 	} else if mergedLayer.Packages == nil {
