@@ -1,13 +1,13 @@
 package rpc
 
 import (
-	"strings"
+	"encoding/json"
 	"time"
 
+	"github.com/package-url/packageurl-go"
 	"github.com/samber/lo"
-	"google.golang.org/protobuf/types/known/timestamppb"
-
 	"google.golang.org/protobuf/types/known/structpb"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	dbTypes "github.com/aquasecurity/trivy-db/pkg/types"
 	"github.com/aquasecurity/trivy/pkg/digest"
@@ -19,38 +19,99 @@ import (
 	"github.com/aquasecurity/trivy/rpc/scanner"
 )
 
+var LicenseCategoryMap = map[common.LicenseCategory_Enum]ftypes.LicenseCategory{
+	common.LicenseCategory_UNSPECIFIED:  "",
+	common.LicenseCategory_FORBIDDEN:    ftypes.CategoryForbidden,
+	common.LicenseCategory_RESTRICTED:   ftypes.CategoryRestricted,
+	common.LicenseCategory_RECIPROCAL:   ftypes.CategoryReciprocal,
+	common.LicenseCategory_NOTICE:       ftypes.CategoryNotice,
+	common.LicenseCategory_PERMISSIVE:   ftypes.CategoryPermissive,
+	common.LicenseCategory_UNENCUMBERED: ftypes.CategoryUnencumbered,
+	common.LicenseCategory_UNKNOWN:      ftypes.CategoryUnknown,
+}
+
+var LicenseTypeMap = map[common.LicenseType_Enum]ftypes.LicenseType{
+	common.LicenseType_UNSPECIFIED:  "",
+	common.LicenseType_DPKG:         ftypes.LicenseTypeDpkg,
+	common.LicenseType_HEADER:       ftypes.LicenseTypeHeader,
+	common.LicenseType_LICENSE_FILE: ftypes.LicenseTypeFile,
+}
+
+// ByValueOr returns the key from the map of the first matched value,
+// or default key if the value is not present.
+func ByValueOr[K, V comparable](m map[K]V, val V, d K) K {
+	for k, v := range m {
+		if v == val {
+			return k
+		}
+	}
+	return d
+}
+
 // ConvertToRPCPkgs returns the list of RPC package objects
 func ConvertToRPCPkgs(pkgs []ftypes.Package) []*common.Package {
 	var rpcPkgs []*common.Package
 	for _, pkg := range pkgs {
 		rpcPkgs = append(rpcPkgs, &common.Package{
-			Id:         pkg.ID,
-			Name:       pkg.Name,
-			Version:    pkg.Version,
-			Release:    pkg.Release,
-			Epoch:      int32(pkg.Epoch),
-			Arch:       pkg.Arch,
-			Dev:        pkg.Dev,
-			SrcName:    pkg.SrcName,
-			SrcVersion: pkg.SrcVersion,
-			SrcRelease: pkg.SrcRelease,
-			SrcEpoch:   int32(pkg.SrcEpoch),
-			Licenses:   pkg.Licenses,
-			Layer:      ConvertToRPCLayer(pkg.Layer),
-			FilePath:   pkg.FilePath,
-			DependsOn:  pkg.DependsOn,
-			Digest:     pkg.Digest.String(),
+			Id:           pkg.ID,
+			Name:         pkg.Name,
+			Version:      pkg.Version,
+			Release:      pkg.Release,
+			Epoch:        int32(pkg.Epoch),
+			Arch:         pkg.Arch,
+			Identifier:   ConvertToRPCPkgIdentifier(pkg.Identifier),
+			Dev:          pkg.Dev,
+			SrcName:      pkg.SrcName,
+			SrcVersion:   pkg.SrcVersion,
+			SrcRelease:   pkg.SrcRelease,
+			SrcEpoch:     int32(pkg.SrcEpoch),
+			Licenses:     pkg.Licenses,
+			Locations:    ConvertToRPCLocations(pkg.Locations),
+			Layer:        ConvertToRPCLayer(pkg.Layer),
+			FilePath:     pkg.FilePath,
+			DependsOn:    pkg.DependsOn,
+			Digest:       pkg.Digest.String(),
+			Relationship: int32(pkg.Relationship),
+			Indirect:     pkg.Indirect,
+			Maintainer:   pkg.Maintainer,
 		})
 	}
 	return rpcPkgs
 }
 
+func ConvertToRPCPkgIdentifier(pkg ftypes.PkgIdentifier) *common.PkgIdentifier {
+	if pkg.Empty() {
+		return nil
+	}
+
+	var p string
+	if pkg.PURL != nil {
+		p = pkg.PURL.String()
+	}
+	return &common.PkgIdentifier{
+		Uid:    pkg.UID,
+		Purl:   p,
+		BomRef: pkg.BOMRef,
+	}
+}
+
+func ConvertToRPCLocations(pkgLocs []ftypes.Location) []*common.Location {
+	var locations []*common.Location
+	for _, pkgLoc := range pkgLocs {
+		locations = append(locations, &common.Location{
+			StartLine: int32(pkgLoc.StartLine),
+			EndLine:   int32(pkgLoc.EndLine),
+		})
+	}
+	return locations
+}
+
 func ConvertToRPCCustomResources(resources []ftypes.CustomResource) []*common.CustomResource {
 	var rpcResources []*common.CustomResource
 	for _, r := range resources {
 		data, err := structpb.NewValue(r.Data)
 		if err != nil {
-			log.Logger.Warn(err)
+			log.Warn("Custom resource conversion error", log.Err(err))
 		}
 		rpcResources = append(rpcResources, &common.CustomResource{
 			Type:     r.Type,
@@ -87,10 +148,7 @@ func ConvertToRPCCode(code ftypes.Code) *common.Code {
 func ConvertToRPCSecrets(secrets []ftypes.Secret) []*common.Secret {
 	var rpcSecrets []*common.Secret
 	for _, s := range secrets {
-		rpcSecrets = append(rpcSecrets, &common.Secret{
-			Filepath: s.FilePath,
-			Findings: ConvertToRPCSecretFindings(s.Findings),
-		})
+		rpcSecrets = append(rpcSecrets, ConvertToRPCSecret(&s))
 	}
 	return rpcSecrets
 }
@@ -113,52 +171,122 @@ func ConvertToRPCSecretFindings(findings []ftypes.SecretFinding) []*common.Secre
 	return rpcFindings
 }
 
+func ConvertToRPCLicenseFiles(licenses []ftypes.LicenseFile) []*common.LicenseFile {
+	var rpcLicenses []*common.LicenseFile
+
+	for _, lic := range licenses {
+		rpcLicenses = append(rpcLicenses, &common.LicenseFile{
+			LicenseType: ConvertToRPCLicenseType(lic.Type),
+			FilePath:    lic.FilePath,
+			PkgName:     lic.PkgName,
+			Fingings:    ConvertToRPCLicenseFindings(lic.Findings),
+			Layer:       ConvertToRPCLayer(lic.Layer),
+		})
+	}
+
+	return rpcLicenses
+}
+
+func ConvertToRPCLicenseFindings(findings ftypes.LicenseFindings) []*common.LicenseFinding {
+	var rpcFindings []*common.LicenseFinding
+
+	for _, f := range findings {
+		rpcFindings = append(rpcFindings, &common.LicenseFinding{
+			Category:   ConvertToRPCLicenseCategory(f.Category),
+			Name:       f.Name,
+			Confidence: float32(f.Confidence),
+			Link:       f.Link,
+		})
+	}
+
+	return rpcFindings
+}
+
 // ConvertFromRPCPkgs returns list of Fanal package objects
 func ConvertFromRPCPkgs(rpcPkgs []*common.Package) []ftypes.Package {
 	var pkgs []ftypes.Package
 	for _, pkg := range rpcPkgs {
 		pkgs = append(pkgs, ftypes.Package{
-			ID:         pkg.Id,
-			Name:       pkg.Name,
-			Version:    pkg.Version,
-			Release:    pkg.Release,
-			Epoch:      int(pkg.Epoch),
-			Arch:       pkg.Arch,
-			Dev:        pkg.Dev,
-			SrcName:    pkg.SrcName,
-			SrcVersion: pkg.SrcVersion,
-			SrcRelease: pkg.SrcRelease,
-			SrcEpoch:   int(pkg.SrcEpoch),
-			Licenses:   pkg.Licenses,
-			Layer:      ConvertFromRPCLayer(pkg.Layer),
-			FilePath:   pkg.FilePath,
-			DependsOn:  pkg.DependsOn,
-			Digest:     digest.Digest(pkg.Digest),
+			ID:           pkg.Id,
+			Name:         pkg.Name,
+			Version:      pkg.Version,
+			Release:      pkg.Release,
+			Epoch:        int(pkg.Epoch),
+			Arch:         pkg.Arch,
+			Identifier:   ConvertFromRPCPkgIdentifier(pkg.Identifier),
+			Dev:          pkg.Dev,
+			SrcName:      pkg.SrcName,
+			SrcVersion:   pkg.SrcVersion,
+			SrcRelease:   pkg.SrcRelease,
+			SrcEpoch:     int(pkg.SrcEpoch),
+			Licenses:     pkg.Licenses,
+			Locations:    ConvertFromRPCLocation(pkg.Locations),
+			Layer:        ConvertFromRPCLayer(pkg.Layer),
+			FilePath:     pkg.FilePath,
+			DependsOn:    pkg.DependsOn,
+			Digest:       digest.Digest(pkg.Digest),
+			Relationship: ftypes.Relationship(pkg.Relationship),
+			Indirect:     pkg.Indirect,
+			Maintainer:   pkg.Maintainer,
 		})
 	}
 	return pkgs
 }
 
+func ConvertFromRPCPkgIdentifier(pkg *common.PkgIdentifier) ftypes.PkgIdentifier {
+	if pkg == nil {
+		return ftypes.PkgIdentifier{}
+	}
+
+	pkgID := ftypes.PkgIdentifier{
+		UID:    pkg.GetUid(),
+		BOMRef: pkg.GetBomRef(),
+	}
+
+	if pkg.Purl != "" {
+		pu, err := packageurl.FromString(pkg.Purl)
+		if err != nil {
+			log.Error("Failed to parse PURL", log.String("purl", pkg.Purl), log.Err(err))
+		}
+		pkgID.PURL = &pu
+	}
+
+	return pkgID
+}
+
+func ConvertFromRPCLocation(locs []*common.Location) []ftypes.Location {
+	var pkgLocs []ftypes.Location
+	for _, loc := range locs {
+		pkgLocs = append(pkgLocs, ftypes.Location{
+			StartLine: int(loc.StartLine),
+			EndLine:   int(loc.EndLine),
+		})
+	}
+	return pkgLocs
+}
+
 // ConvertToRPCVulns returns common.Vulnerability
 func ConvertToRPCVulns(vulns []types.DetectedVulnerability) []*common.Vulnerability {
 	var rpcVulns []*common.Vulnerability
 	for _, vuln := range vulns {
 		severity, err := dbTypes.NewSeverity(vuln.Severity)
 		if err != nil {
-			log.Logger.Warn(err)
+			log.Warn("Severity error", log.Err(err))
 		}
 		cvssMap := make(map[string]*common.CVSS) // This is needed because protobuf generates a map[string]*CVSS type
 		for vendor, vendorSeverity := range vuln.CVSS {
 			cvssMap[string(vendor)] = &common.CVSS{
-				V2Vector: vendorSeverity.V2Vector,
-				V3Vector: vendorSeverity.V3Vector,
-				V2Score:  vendorSeverity.V2Score,
-				V3Score:  vendorSeverity.V3Score,
+				V2Vector:  vendorSeverity.V2Vector,
+				V3Vector:  vendorSeverity.V3Vector,
+				V40Vector: vendorSeverity.V40Vector,
+				V2Score:   vendorSeverity.V2Score,
+				V3Score:   vendorSeverity.V3Score,
+				V40Score:  vendorSeverity.V40Score,
 			}
 		}
-		vensorSeverityMap := make(map[string]common.Severity)
+		vendorSeverityMap := make(map[string]common.Severity)
 		for vendor, vendorSeverity := range vuln.VendorSeverity {
-			vensorSeverityMap[string(vendor)] = common.Severity(vendorSeverity)
+			vendorSeverityMap[string(vendor)] = common.Severity(vendorSeverity)
 		}
 
 		var lastModifiedDate, publishedDate *timestamppb.Timestamp
@@ -170,12 +298,14 @@ func ConvertToRPCVulns(vulns []types.DetectedVulnerability) []*common.Vulnerabil
 			publishedDate = timestamppb.New(*vuln.PublishedDate) // nolint: errcheck
 		}
 
-		var customAdvisoryData, customVulnData *structpb.Value
+		var customAdvisoryData, customVulnData []byte
 		if vuln.Custom != nil {
-			customAdvisoryData, _ = structpb.NewValue(vuln.Custom) // nolint: errcheck
+			jsonBytes, _ := json.Marshal(vuln.Custom) // nolint: errcheck
+			customAdvisoryData = jsonBytes
 		}
 		if vuln.Vulnerability.Custom != nil {
-			customVulnData, _ = structpb.NewValue(vuln.Vulnerability.Custom) // nolint: errcheck
+			jsonBytes, _ := json.Marshal(vuln.Vulnerability.Custom) // nolint: errcheck
+			customVulnData = jsonBytes
 		}
 
 		rpcVulns = append(rpcVulns, &common.Vulnerability{
@@ -186,11 +316,12 @@ func ConvertToRPCVulns(vulns []types.DetectedVulnerability) []*common.Vulnerabil
 			PkgPath:            vuln.PkgPath,
 			InstalledVersion:   vuln.InstalledVersion,
 			FixedVersion:       vuln.FixedVersion,
+			PkgIdentifier:      ConvertToRPCPkgIdentifier(vuln.PkgIdentifier),
 			Status:             int32(vuln.Status),
 			Title:              vuln.Title,
 			Description:        vuln.Description,
 			Severity:           common.Severity(severity),
-			VendorSeverity:     vensorSeverityMap,
+			VendorSeverity:     vendorSeverityMap,
 			References:         vuln.References,
 			Layer:              ConvertToRPCLayer(vuln.Layer),
 			Cvss:               cvssMap,
@@ -213,7 +344,7 @@ func ConvertToRPCMisconfs(misconfs []types.DetectedMisconfiguration) []*common.D
 	for _, m := range misconfs {
 		severity, err := dbTypes.NewSeverity(m.Severity)
 		if err != nil {
-			log.Logger.Warn(err)
+			log.Warn("Severity conversion error", log.Err(err))
 		}
 
 		rpcMisconfs = append(rpcMisconfs, &common.DetectedMisconfiguration{
@@ -240,6 +371,7 @@ func ConvertToRPCMisconfs(misconfs []types.DetectedMisconfiguration) []*common.D
 // ConvertToRPCLayer returns common.Layer
 func ConvertToRPCLayer(layer ftypes.Layer) *common.Layer {
 	return &common.Layer{
+		Size:      layer.Size,
 		Digest:    layer.Digest,
 		DiffId:    layer.DiffID,
 		CreatedBy: layer.CreatedBy,
@@ -267,6 +399,10 @@ func ConvertToRPCCauseMetadata(cause ftypes.CauseMetadata) *common.CauseMetadata
 		StartLine: int32(cause.StartLine),
 		EndLine:   int32(cause.EndLine),
 		Code:      ConvertToRPCCode(cause.Code),
+		RenderedCause: &common.RenderedCause{
+			Raw:         cause.RenderedCause.Raw,
+			Highlighted: cause.RenderedCause.Highlighted,
+		},
 	}
 }
 
@@ -291,17 +427,17 @@ func ConvertFromRPCResults(rpcResults []*scanner.Result) []types.Result {
 			Vulnerabilities:   ConvertFromRPCVulns(result.Vulnerabilities),
 			Misconfigurations: ConvertFromRPCMisconfs(result.Misconfigurations),
 			Class:             types.ResultClass(result.Class),
-			Type:              result.Type,
+			Type:              ftypes.TargetType(result.Type),
 			Packages:          ConvertFromRPCPkgs(result.Packages),
 			CustomResources:   ConvertFromRPCCustomResources(result.CustomResources),
-			Secrets:           ConvertFromRPCSecretFindings(result.Secrets),
-			Licenses:          ConvertFromRPCLicenses(result.Licenses),
+			Secrets:           ConvertFromRPCDetectedSecrets(result.Secrets),
+			Licenses:          ConvertFromRPCDetectedLicenses(result.Licenses),
 		})
 	}
 	return results
 }
 
-func ConvertFromRPCLicenses(rpcLicenses []*common.DetectedLicense) []types.DetectedLicense {
+func ConvertFromRPCDetectedLicenses(rpcLicenses []*common.DetectedLicense) []types.DetectedLicense {
 	var licenses []types.DetectedLicense
 	for _, l := range rpcLicenses {
 		severity := dbTypes.Severity(l.Severity)
@@ -311,6 +447,7 @@ func ConvertFromRPCLicenses(rpcLicenses []*common.DetectedLicense) []types.Detec
 			PkgName:    l.PkgName,
 			FilePath:   l.FilePath,
 			Name:       l.Name,
+			Text:       l.Text,
 			Confidence: float64(l.Confidence),
 			Link:       l.Link,
 		})
@@ -318,11 +455,12 @@ func ConvertFromRPCLicenses(rpcLicenses []*common.DetectedLicense) []types.Detec
 	return licenses
 }
 
-func ConvertFromRPCLicenseCategory(rpcCategory common.DetectedLicense_LicenseCategory) ftypes.LicenseCategory {
-	if rpcCategory == common.DetectedLicense_UNSPECIFIED {
-		return ""
-	}
-	return ftypes.LicenseCategory(strings.ToLower(rpcCategory.String()))
+func ConvertFromRPCLicenseCategory(rpcCategory common.LicenseCategory_Enum) ftypes.LicenseCategory {
+	return lo.ValueOr(LicenseCategoryMap, rpcCategory, "")
+}
+
+func ConvertFromRPCLicenseType(rpcLicenseType common.LicenseType_Enum) ftypes.LicenseType {
+	return lo.ValueOr(LicenseTypeMap, rpcLicenseType, "")
 }
 
 // ConvertFromRPCCustomResources converts array of cache.CustomResource to fanal.CustomResource
@@ -361,6 +499,15 @@ func ConvertFromRPCCode(rpcCode *common.Code) ftypes.Code {
 	}
 }
 
+func ConvertFromRPCDetectedSecrets(rpcFindings []*common.SecretFinding) []types.DetectedSecret {
+	if len(rpcFindings) == 0 {
+		return nil
+	}
+	return lo.Map(ConvertFromRPCSecretFindings(rpcFindings), func(s ftypes.SecretFinding, _ int) types.DetectedSecret {
+		return types.DetectedSecret(s)
+	})
+}
+
 func ConvertFromRPCSecretFindings(rpcFindings []*common.SecretFinding) []ftypes.SecretFinding {
 	var findings []ftypes.SecretFinding
 	for _, finding := range rpcFindings {
@@ -385,15 +532,43 @@ func ConvertFromRPCSecretFindings(rpcFindings []*common.SecretFinding) []ftypes.
 
 func ConvertFromRPCSecrets(recSecrets []*common.Secret) []ftypes.Secret {
 	var secrets []ftypes.Secret
-	for _, secret := range recSecrets {
-		secrets = append(secrets, ftypes.Secret{
-			FilePath: secret.Filepath,
-			Findings: ConvertFromRPCSecretFindings(secret.Findings),
-		})
+	for _, recSecret := range recSecrets {
+		secrets = append(secrets, *ConvertFromRPCSecret(recSecret))
 	}
 	return secrets
 }
 
+func ConvertFromRPCLicenseFiles(rpcLicenses []*common.LicenseFile) []ftypes.LicenseFile {
+	var licenses []ftypes.LicenseFile
+
+	for _, lic := range rpcLicenses {
+		licenses = append(licenses, ftypes.LicenseFile{
+			Type:     ConvertFromRPCLicenseType(lic.LicenseType),
+			FilePath: lic.FilePath,
+			PkgName:  lic.PkgName,
+			Findings: ConvertFromRPCLicenseFindings(lic.Fingings),
+			Layer:    ConvertFromRPCLayer(lic.Layer),
+		})
+	}
+
+	return licenses
+}
+
+func ConvertFromRPCLicenseFindings(rpcFindings []*common.LicenseFinding) ftypes.LicenseFindings {
+	var findings ftypes.LicenseFindings
+
+	for _, finding := range rpcFindings {
+		findings = append(findings, ftypes.LicenseFinding{
+			Category:   ConvertFromRPCLicenseCategory(finding.Category),
+			Name:       finding.Name,
+			Confidence: float64(finding.Confidence),
+			Link:       finding.Link,
+		})
+	}
+
+	return findings
+}
+
 // ConvertFromRPCVulns converts []*common.Vulnerability to []types.DetectedVulnerability
 func ConvertFromRPCVulns(rpcVulns []*common.Vulnerability) []types.DetectedVulnerability {
 	var vulns []types.DetectedVulnerability
@@ -402,15 +577,17 @@ func ConvertFromRPCVulns(rpcVulns []*common.Vulnerability) []types.DetectedVulne
 		cvssMap := make(dbTypes.VendorCVSS) // This is needed because protobuf generates a map[string]*CVSS type
 		for vendor, vendorSeverity := range vuln.Cvss {
 			cvssMap[dbTypes.SourceID(vendor)] = dbTypes.CVSS{
-				V2Vector: vendorSeverity.V2Vector,
-				V3Vector: vendorSeverity.V3Vector,
-				V2Score:  vendorSeverity.V2Score,
-				V3Score:  vendorSeverity.V3Score,
+				V2Vector:  vendorSeverity.V2Vector,
+				V3Vector:  vendorSeverity.V3Vector,
+				V40Vector: vendorSeverity.V40Vector,
+				V2Score:   vendorSeverity.V2Score,
+				V3Score:   vendorSeverity.V3Score,
+				V40Score:  vendorSeverity.V40Score,
 			}
 		}
-		vensorSeverityMap := make(dbTypes.VendorSeverity)
+		vendorSeverityMap := make(dbTypes.VendorSeverity)
 		for vendor, vendorSeverity := range vuln.VendorSeverity {
-			vensorSeverityMap[dbTypes.SourceID(vendor)] = dbTypes.Severity(vendorSeverity)
+			vendorSeverityMap[dbTypes.SourceID(vendor)] = dbTypes.Severity(vendorSeverity)
 		}
 
 		var lastModifiedDate, publishedDate *time.Time
@@ -429,6 +606,7 @@ func ConvertFromRPCVulns(rpcVulns []*common.Vulnerability) []types.DetectedVulne
 			PkgPath:          vuln.PkgPath,
 			InstalledVersion: vuln.InstalledVersion,
 			FixedVersion:     vuln.FixedVersion,
+			PkgIdentifier:    ConvertFromRPCPkgIdentifier(vuln.PkgIdentifier),
 			Status:           dbTypes.Status(vuln.Status),
 			Vulnerability: dbTypes.Vulnerability{
 				Title:            vuln.Title,
@@ -439,13 +617,13 @@ func ConvertFromRPCVulns(rpcVulns []*common.Vulnerability) []types.DetectedVulne
 				CweIDs:           vuln.CweIds,
 				LastModifiedDate: lastModifiedDate,
 				PublishedDate:    publishedDate,
-				Custom:           vuln.CustomVulnData.AsInterface(),
-				VendorSeverity:   vensorSeverityMap,
+				Custom:           vuln.CustomVulnData,
+				VendorSeverity:   vendorSeverityMap,
 			},
 			Layer:          ConvertFromRPCLayer(vuln.Layer),
 			SeveritySource: dbTypes.SourceID(vuln.SeveritySource),
 			PrimaryURL:     vuln.PrimaryUrl,
-			Custom:         vuln.CustomAdvisoryData.AsInterface(),
+			Custom:         vuln.CustomAdvisoryData,
 			DataSource:     ConvertFromRPCDataSource(vuln.DataSource),
 		})
 	}
@@ -477,12 +655,13 @@ func ConvertFromRPCMisconfs(rpcMisconfs []*common.DetectedMisconfiguration) []ty
 	return misconfs
 }
 
-// ConvertFromRPCLayer converts *common.Layer to fanal.Layer
+// ConvertFromRPCLayer converts *common.Layer to ftypes.Layer
 func ConvertFromRPCLayer(rpcLayer *common.Layer) ftypes.Layer {
 	if rpcLayer == nil {
 		return ftypes.Layer{}
 	}
 	return ftypes.Layer{
+		Size:      rpcLayer.Size,
 		Digest:    rpcLayer.Digest,
 		DiffID:    rpcLayer.DiffId,
 		CreatedBy: rpcLayer.CreatedBy,
@@ -511,12 +690,23 @@ func ConvertFromRPCCauseMetadata(rpcCause *common.CauseMetadata) ftypes.CauseMet
 		return ftypes.CauseMetadata{}
 	}
 	return ftypes.CauseMetadata{
-		Resource:  rpcCause.Resource,
-		Provider:  rpcCause.Provider,
-		Service:   rpcCause.Service,
-		StartLine: int(rpcCause.StartLine),
-		EndLine:   int(rpcCause.EndLine),
-		Code:      ConvertFromRPCCode(rpcCause.Code),
+		Resource:      rpcCause.Resource,
+		Provider:      rpcCause.Provider,
+		Service:       rpcCause.Service,
+		StartLine:     int(rpcCause.StartLine),
+		EndLine:       int(rpcCause.EndLine),
+		Code:          ConvertFromRPCCode(rpcCause.Code),
+		RenderedCause: ConvertFromRPCRenderedCause(rpcCause.RenderedCause),
+	}
+}
+
+func ConvertFromRPCRenderedCause(rendered *common.RenderedCause) ftypes.RenderedCause {
+	if rendered == nil {
+		return ftypes.RenderedCause{}
+	}
+	return ftypes.RenderedCause{
+		Raw:         rendered.Raw,
+		Highlighted: rendered.Highlighted,
 	}
 }
 
@@ -526,7 +716,7 @@ func ConvertFromRPCOS(rpcOS *common.OS) ftypes.OS {
 		return ftypes.OS{}
 	}
 	return ftypes.OS{
-		Family:   rpcOS.Family,
+		Family:   ftypes.OSType(rpcOS.Family),
 		Name:     rpcOS.Name,
 		Eosl:     rpcOS.Eosl,
 		Extended: rpcOS.Extended,
@@ -539,7 +729,7 @@ func ConvertFromRPCRepository(rpcRepo *common.Repository) *ftypes.Repository {
 		return nil
 	}
 	return &ftypes.Repository{
-		Family:  rpcRepo.Family,
+		Family:  ftypes.OSType(rpcRepo.Family),
 		Release: rpcRepo.Release,
 	}
 }
@@ -573,9 +763,9 @@ func ConvertFromRPCApplications(rpcApps []*common.Application) []ftypes.Applicat
 	var apps []ftypes.Application
 	for _, rpcApp := range rpcApps {
 		apps = append(apps, ftypes.Application{
-			Type:      rpcApp.Type,
-			FilePath:  rpcApp.FilePath,
-			Libraries: ConvertFromRPCPkgs(rpcApp.Libraries),
+			Type:     ftypes.LangType(rpcApp.Type),
+			FilePath: rpcApp.FilePath,
+			Packages: ConvertFromRPCPkgs(rpcApp.Packages),
 		})
 	}
 	return apps
@@ -586,13 +776,12 @@ func ConvertFromRPCMisconfigurations(rpcMisconfs []*common.Misconfiguration) []f
 	var misconfs []ftypes.Misconfiguration
 	for _, rpcMisconf := range rpcMisconfs {
 		misconfs = append(misconfs, ftypes.Misconfiguration{
-			FileType:   rpcMisconf.FileType,
-			FilePath:   rpcMisconf.FilePath,
-			Successes:  ConvertFromRPCMisconfResults(rpcMisconf.Successes),
-			Warnings:   ConvertFromRPCMisconfResults(rpcMisconf.Warnings),
-			Failures:   ConvertFromRPCMisconfResults(rpcMisconf.Failures),
-			Exceptions: ConvertFromRPCMisconfResults(rpcMisconf.Exceptions),
-			Layer:      ftypes.Layer{},
+			FileType:  ftypes.ConfigType(rpcMisconf.FileType),
+			FilePath:  rpcMisconf.FilePath,
+			Successes: ConvertFromRPCMisconfResults(rpcMisconf.Successes),
+			Warnings:  ConvertFromRPCMisconfResults(rpcMisconf.Warnings),
+			Failures:  ConvertFromRPCMisconfResults(rpcMisconf.Failures),
+			Layer:     ftypes.Layer{},
 		})
 	}
 	return misconfs
@@ -621,6 +810,7 @@ func ConvertFromRPCPutArtifactRequest(req *cache.PutArtifactRequest) ftypes.Arti
 		DockerVersion:   req.ArtifactInfo.DockerVersion,
 		OS:              req.ArtifactInfo.Os,
 		HistoryPackages: ConvertFromRPCPkgs(req.ArtifactInfo.HistoryPackages),
+		Secret:          ConvertFromRPCSecret(req.ArtifactInfo.Secret),
 	}
 }
 
@@ -628,6 +818,7 @@ func ConvertFromRPCPutArtifactRequest(req *cache.PutArtifactRequest) ftypes.Arti
 func ConvertFromRPCPutBlobRequest(req *cache.PutBlobRequest) ftypes.BlobInfo {
 	return ftypes.BlobInfo{
 		SchemaVersion:     int(req.BlobInfo.SchemaVersion),
+		Size:              req.BlobInfo.Size,
 		Digest:            req.BlobInfo.Digest,
 		DiffID:            req.BlobInfo.DiffId,
 		OS:                ConvertFromRPCOS(req.BlobInfo.Os),
@@ -635,17 +826,19 @@ func ConvertFromRPCPutBlobRequest(req *cache.PutBlobRequest) ftypes.BlobInfo {
 		PackageInfos:      ConvertFromRPCPackageInfos(req.BlobInfo.PackageInfos),
 		Applications:      ConvertFromRPCApplications(req.BlobInfo.Applications),
 		Misconfigurations: ConvertFromRPCMisconfigurations(req.BlobInfo.Misconfigurations),
-		OpaqueDirs:        req.BlobInfo.OpaqueDirs,
-		WhiteoutFiles:     req.BlobInfo.WhiteoutFiles,
 		CustomResources:   ConvertFromRPCCustomResources(req.BlobInfo.CustomResources),
 		Secrets:           ConvertFromRPCSecrets(req.BlobInfo.Secrets),
+		Licenses:          ConvertFromRPCLicenseFiles(req.BlobInfo.Licenses),
+		CreatedBy:         req.BlobInfo.CreatedBy,
+		OpaqueDirs:        req.BlobInfo.OpaqueDirs,
+		WhiteoutFiles:     req.BlobInfo.WhiteoutFiles,
 	}
 }
 
 // ConvertToRPCOS returns common.OS
 func ConvertToRPCOS(fos ftypes.OS) *common.OS {
 	return &common.OS{
-		Family:   fos.Family,
+		Family:   string(fos.Family),
 		Name:     fos.Name,
 		Eosl:     fos.Eosl,
 		Extended: fos.Extended,
@@ -658,7 +851,7 @@ func ConvertToRPCRepository(repo *ftypes.Repository) *common.Repository {
 		return nil
 	}
 	return &common.Repository{
-		Family:  repo.Family,
+		Family:  string(repo.Family),
 		Release: repo.Release,
 	}
 }
@@ -668,7 +861,7 @@ func ConvertToRPCArtifactInfo(imageID string, imageInfo ftypes.ArtifactInfo) *ca
 
 	t := timestamppb.New(imageInfo.Created)
 	if err := t.CheckValid(); err != nil {
-		log.Logger.Warnf("invalid timestamp: %s", err)
+		log.Warn("Invalid timestamp", log.Err(err))
 	}
 
 	return &cache.PutArtifactRequest{
@@ -680,6 +873,7 @@ func ConvertToRPCArtifactInfo(imageID string, imageInfo ftypes.ArtifactInfo) *ca
 			DockerVersion:   imageInfo.DockerVersion,
 			Os:              imageInfo.OS,
 			HistoryPackages: ConvertToRPCPkgs(imageInfo.HistoryPackages),
+			Secret:          ConvertToRPCSecret(imageInfo.Secret),
 		},
 	}
 }
@@ -697,21 +891,20 @@ func ConvertToRPCPutBlobRequest(diffID string, blobInfo ftypes.BlobInfo) *cache.
 	var applications []*common.Application
 	for _, app := range blobInfo.Applications {
 		applications = append(applications, &common.Application{
-			Type:      app.Type,
-			FilePath:  app.FilePath,
-			Libraries: ConvertToRPCPkgs(app.Libraries),
+			Type:     string(app.Type),
+			FilePath: app.FilePath,
+			Packages: ConvertToRPCPkgs(app.Packages),
 		})
 	}
 
 	var misconfigurations []*common.Misconfiguration
 	for _, m := range blobInfo.Misconfigurations {
 		misconfigurations = append(misconfigurations, &common.Misconfiguration{
-			FileType:   m.FileType,
-			FilePath:   m.FilePath,
-			Successes:  ConvertToMisconfResults(m.Successes),
-			Warnings:   ConvertToMisconfResults(m.Warnings),
-			Failures:   ConvertToMisconfResults(m.Failures),
-			Exceptions: ConvertToMisconfResults(m.Exceptions),
+			FileType:  string(m.FileType),
+			FilePath:  m.FilePath,
+			Successes: ConvertToMisconfResults(m.Successes),
+			Warnings:  ConvertToMisconfResults(m.Warnings),
+			Failures:  ConvertToMisconfResults(m.Failures),
 		})
 
 	}
@@ -738,6 +931,7 @@ func ConvertToRPCPutBlobRequest(diffID string, blobInfo ftypes.BlobInfo) *cache.
 		DiffId: diffID,
 		BlobInfo: &cache.BlobInfo{
 			SchemaVersion:     ftypes.BlobJSONSchemaVersion,
+			Size:              blobInfo.Size,
 			Digest:            blobInfo.Digest,
 			DiffId:            blobInfo.DiffID,
 			Os:                ConvertToRPCOS(blobInfo.OS),
@@ -745,10 +939,12 @@ func ConvertToRPCPutBlobRequest(diffID string, blobInfo ftypes.BlobInfo) *cache.
 			PackageInfos:      packageInfos,
 			Applications:      applications,
 			Misconfigurations: misconfigurations,
-			OpaqueDirs:        blobInfo.OpaqueDirs,
-			WhiteoutFiles:     blobInfo.WhiteoutFiles,
 			CustomResources:   customResources,
 			Secrets:           ConvertToRPCSecrets(blobInfo.Secrets),
+			Licenses:          ConvertToRPCLicenseFiles(blobInfo.Licenses),
+			CreatedBy:         blobInfo.CreatedBy,
+			OpaqueDirs:        blobInfo.OpaqueDirs,
+			WhiteoutFiles:     blobInfo.WhiteoutFiles,
 		},
 	}
 }
@@ -776,25 +972,31 @@ func ConvertToMissingBlobsRequest(imageID string, layerIDs []string) *cache.Miss
 }
 
 // ConvertToRPCScanResponse converts types.Result to ScanResponse
-func ConvertToRPCScanResponse(results types.Results, fos ftypes.OS) *scanner.ScanResponse {
+func ConvertToRPCScanResponse(response types.ScanResponse) *scanner.ScanResponse {
 	var rpcResults []*scanner.Result
-	for _, result := range results {
+	for _, result := range response.Results {
+		secretFindings := lo.Map(result.Secrets, func(s types.DetectedSecret, _ int) ftypes.SecretFinding {
+			return ftypes.SecretFinding(s)
+		})
 		rpcResults = append(rpcResults, &scanner.Result{
 			Target:            result.Target,
 			Class:             string(result.Class),
-			Type:              result.Type,
+			Type:              string(result.Type),
+			Packages:          ConvertToRPCPkgs(result.Packages),
 			Vulnerabilities:   ConvertToRPCVulns(result.Vulnerabilities),
 			Misconfigurations: ConvertToRPCMisconfs(result.Misconfigurations),
-			Packages:          ConvertToRPCPkgs(result.Packages),
-			CustomResources:   ConvertToRPCCustomResources(result.CustomResources),
-			Secrets:           ConvertToRPCSecretFindings(result.Secrets),
+			Secrets:           ConvertToRPCSecretFindings(secretFindings),
 			Licenses:          ConvertToRPCLicenses(result.Licenses),
+			CustomResources:   ConvertToRPCCustomResources(result.CustomResources),
 		})
 	}
 
 	return &scanner.ScanResponse{
-		Os:      ConvertToRPCOS(fos),
+		Os:      ConvertToRPCOS(response.OS),
 		Results: rpcResults,
+		Layers: lo.Map(response.Layers, func(layer ftypes.Layer, _ int) *common.Layer {
+			return ConvertToRPCLayer(layer)
+		}),
 	}
 }
 
@@ -803,7 +1005,7 @@ func ConvertToRPCLicenses(licenses []types.DetectedLicense) []*common.DetectedLi
 	for _, l := range licenses {
 		severity, err := dbTypes.NewSeverity(l.Severity)
 		if err != nil {
-			log.Logger.Warn(err)
+			log.Warn("Severity conversion error", log.Err(err))
 		}
 		rpcLicenses = append(rpcLicenses, &common.DetectedLicense{
 			Severity:   common.Severity(severity),
@@ -811,6 +1013,7 @@ func ConvertToRPCLicenses(licenses []types.DetectedLicense) []*common.DetectedLi
 			PkgName:    l.PkgName,
 			FilePath:   l.FilePath,
 			Name:       l.Name,
+			Text:       l.Text,
 			Confidence: float32(l.Confidence),
 			Link:       l.Link,
 		})
@@ -819,12 +1022,12 @@ func ConvertToRPCLicenses(licenses []types.DetectedLicense) []*common.DetectedLi
 	return rpcLicenses
 }
 
-func ConvertToRPCLicenseCategory(category ftypes.LicenseCategory) common.DetectedLicense_LicenseCategory {
-	rpcCategory, ok := common.DetectedLicense_LicenseCategory_value[strings.ToUpper(string(category))]
-	if !ok {
-		return common.DetectedLicense_UNSPECIFIED
-	}
-	return common.DetectedLicense_LicenseCategory(rpcCategory)
+func ConvertToRPCLicenseCategory(category ftypes.LicenseCategory) common.LicenseCategory_Enum {
+	return ByValueOr(LicenseCategoryMap, category, common.LicenseCategory_UNSPECIFIED)
+}
+
+func ConvertToRPCLicenseType(ty ftypes.LicenseType) common.LicenseType_Enum {
+	return ByValueOr(LicenseTypeMap, ty, common.LicenseType_UNSPECIFIED)
 }
 
 func ConvertToDeleteBlobsRequest(blobIDs []string) *cache.DeleteBlobsRequest {
@@ -837,3 +1040,25 @@ func ConvertFromDeleteBlobsRequest(deleteBlobsRequest *cache.DeleteBlobsRequest)
 	}
 	return deleteBlobsRequest.GetBlobIds()
 }
+
+// ConvertFromRPCSecret converts common.Secret to fanal.Secret
+func ConvertFromRPCSecret(rpcSecret *common.Secret) *ftypes.Secret {
+	if rpcSecret == nil {
+		return nil
+	}
+	return &ftypes.Secret{
+		FilePath: rpcSecret.Filepath,
+		Findings: ConvertFromRPCSecretFindings(rpcSecret.Findings),
+	}
+}
+
+// ConvertToRPCSecret converts fanal.Secret to common.Secret
+func ConvertToRPCSecret(secret *ftypes.Secret) *common.Secret {
+	if secret == nil {
+		return nil
+	}
+	return &common.Secret{
+		Filepath: secret.FilePath,
+		Findings: ConvertToRPCSecretFindings(secret.Findings),
+	}
+}
